/*
 *   Copyright (C) 2010 Casey Link <unnamedrambler@gmail.com>
 *   Copyright (c) 2009-2010 Klaralvdalens Datakonsult AB, a KDAB Group company <info@kdab.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#ifndef TESTIDXREADER_DATA
#define TESTIDXREADER_DATA

/* contains one maildir with one  email inside*/
static const char mailDirOneEmail[]
    = "IyBLTWFpbC1JbmRleCBWMTUwNgoACAAAAHhWNBIIAAAAagEAAAUAAAAAAAgAAAAIAAAAAAAAAAAA"
      "AQAAABQAAEMAYQBzAGUAeQAgAEwAaQBuAGsTAAAAAAACAAAAIAAAaABlAGwAbABvACAAZgByAG8A"
      "bQAgAGsAbQBhAGkAbAMAAAAcAABLAEQAQQBCACAARQBtAHAAbABvAHkAZQBlAHMEAAAAAAAGAAAA"
      "AAALAAAANAAAMQAyADcAMQA1ADEAMwA1ADMANwAuADEAOQA1ADcANgAuADEATABLADYAZQA6ADIA"
      "LABTCQAAAAgAhAIAAAAAAAAHAAAACAAAAAAAAAAAAAoAAAAIAMHByUsAAAAADAAAAAgATgBOAAAA"
      "AAANAAAACAAgAAAAAAAAAA4AAAAAAA8AAAAsAABxADEAUABJAFkAbgBhAGwAVwB3AFYAQgAzAHEA"
      "aAB2AEYAUABUAHQAaQBBEAAAAAgABAAAAAAAAAARAAAACAAAAAAAAAAAABIAAAAIAAAAAAAAAAAA";

/*
 Folder: "Test1" type:Maildir
 Message1: "foo bar"  Important, Unread
 Message2: "hello from kmail" Read, tagged "N5tUHPOZFf"
 */
static const char mailDirTwoEmailOneTagFlags[]
    = "IyBLTWFpbC1JbmRleCBWMTUwNgoACAAAAHhWNBIIAAAAlAEAAAUAAAAAAAgAAAAIAAAAAAAAAAAA"
      "AQAAABQAAEMAYQBzAGUAeQAgAEwAaQBuAGsTAAAAKgAATgA1AHQAVQBIAFAATwBaAEYAZgAsAFUA"
      "RgBNAEUAagBHAHMAUQA5AG8CAAAAIAAAaABlAGwAbABvACAAZgByAG8AbQAgAGsAbQBhAGkAbAMA"
      "AAAcAABLAEQAQQBCACAARQBtAHAAbABvAHkAZQBlAHMEAAAAAAAGAAAAAAALAAAANAAAMQAyADcA"
      "MQA1ADEAMwA1ADMANwAuADEAOQA1ADcANgAuADEATABLADYAZQA6ADIALABTCQAAAAgAhAIAAAAA"
      "AAAHAAAACAAAAAAAAAAAAAoAAAAIAMHByUsAAAAADAAAAAgATgBOAAAAAAANAAAACAAgAAAAAAAA"
      "AA4AAAAAAA8AAAAsAABxADEAUABJAFkAbgBhAGwAVwB3AFYAQgAzAHEAaAB2AEYAUABUAHQAaQBB"
      "EAAAAAgABAAAAAAAAAARAAAACAAAAAAAAAAAABIAAAAIAAAAAAAAAAAASAEAAAUAAAAAAAgAAAAI"
      "AAAAAAAAAAAAAQAAABQAAEMAYQBzAGUAeQAgAEwAaQBuAGsTAAAAAAACAAAADgAAZgBvAG8AIABi"
      "AGEAcgMAAAAWAABvAG4AZQBAAG8AbgBlAC4AYwBvAG0EAAAAAAAGAAAAAAALAAAAKgAAMQAyADcA"
      "MQA3ADcAMwAwADEAOQAuADUANwA1ADQALgBwAFcAcQBaAFMJAAAACABKAgAAAAAAAAcAAAAIAAAA"
      "AAAAAAAACgAAAAgAW7fNSwAAAAAMAAAACABOAE4AAAAAAA0AAAAIACAAAAAAAAAADgAAAAAADwAA"
      "ACwAAE0AbgB0AHYAQgAwAE4AWQBFAFMATwBiAHgASAA0AFYAUgBEAFUAeQBjAHcQAAAACAACAgAA"
      "AAAAABEAAAAIAAAAAAAAAAAAEgAAAAgAAAAAAAAAAAA=";

#endif
