/*
    This file is part of oxaccess.

    Copyright (c) 2012 Marco Nelles <marco.nelles@credativ.com>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#ifndef OXA_OXERRORS_H
#define OXA_OXERRORS_H

#include <QString>

namespace OXA {
/**
 * Namespace that contains methods for handling OX errors.
 *
 * @author Marco Nelles <marco.nelles@credativ.com>
 */
namespace OXErrors {
enum EditErrorID {
    EditErrorUndefined = 0,
    ConcurrentModification,
    ObjectNotFound,
    NoPermissionForThisAction,
    ConflictsDetected,
    MissingMandatoryFields,
    AppointmentConflicts,
    InternalServerError
};

/**
 * Parse error id from edit error text string @p errorText
 */
EditErrorID getEditErrorID(const QString &errorText);
}
}

#endif
