/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef UPGRADETEST_H
#define UPGRADETEST_H

#include <QObject>

/**
 * Test format upgrade from v2 to v3
 *
 * - Parse using v2 implementation
 * - Serialize and reparse using v3 implementation
 * - Compare results
 *
 * This ensures that all properties which are interpreted by the v2 implementation can be serialized and deserialized by the v3 implementation.
 *
 * Depends on serialization used for comparison to be correct (i.e. the ical implementation).
 */

class UpgradeTest : public QObject
{
    Q_OBJECT
private Q_SLOTS:

    void testIncidence_data();
    void testIncidence();

    void testContact_data();
    void testContact();
};

#endif // UPGRADETEST_H
